
import React, { lazy, Suspense } from 'react'
import { createRoot } from 'react-dom/client'
import './index.css'
import './styles/animations.css'

// Importación diferida del componente App para mejorar el rendimiento inicial
const App = lazy(() => import('./App.tsx'));

// Componente de carga mientras se inicializa la aplicación
const AppLoader = () => (
  <div className="flex h-screen w-screen items-center justify-center bg-gradient-to-br from-auralis-dark via-auralis to-auralis-light">
    <div className="text-center">
      <div className="h-16 w-16 mx-auto mb-4 rounded-full border-4 border-white border-t-transparent animate-spin"></div>
      <p className="text-white">Cargando Auralis...</p>
    </div>
  </div>
);

// Implementar mecanismo de hidratación progresiva para mejorar TTI
const root = document.getElementById('root');

if (!root) {
  throw new Error('No se encontró el elemento root');
}

// Inicializar la aplicación con un enfoque de renderizado progresivo
const renderApp = () => {
  // Marcar el tiempo de inicio para métricas
  if ('performance' in window) {
    performance.mark('render-start');
  }
  
  const appRoot = createRoot(root);
  
  appRoot.render(
    <React.StrictMode>
      <Suspense fallback={<AppLoader />}>
        <App />
      </Suspense>
    </React.StrictMode>
  );
  
  // Marcar el tiempo de renderizado para métricas
  if ('performance' in window) {
    performance.mark('app-rendered');
    performance.measure('time-to-render', 'render-start', 'app-rendered');
    
    // Registrar métrica en console para debugging
    const perfEntry = performance.getEntriesByName('time-to-render')[0];
    console.log(`App renderizada en: ${Math.round(perfEntry.duration)}ms`);
  }
};

// Mejorar FCP detectando la visibilidad de la página
if (document.visibilityState === 'visible') {
  renderApp();
} else {
  // Si la página no es visible (pestaña en segundo plano),
  // esperar hasta que sea visible para renderizar
  const handleVisibilityChange = () => {
    if (document.visibilityState === 'visible') {
      renderApp();
      document.removeEventListener('visibilitychange', handleVisibilityChange);
    }
  };
  
  document.addEventListener('visibilitychange', handleVisibilityChange);
  
  // Asegurarse de renderizar después de un tiempo máximo
  setTimeout(renderApp, 1500);
}

// Reportar Web Vitals
if (process.env.NODE_ENV === 'production') {
  // Medir Core Web Vitals
  const reportWebVitals = () => {
    if ('PerformanceObserver' in window) {
      try {
        // LCP
        new PerformanceObserver((entryList) => {
          const entries = entryList.getEntries();
          if (entries.length > 0) {
            const lcp = entries[entries.length - 1];
            console.log('LCP:', Math.round(lcp.startTime), 'ms');
          }
        }).observe({ type: 'largest-contentful-paint', buffered: true });
        
        // FID
        new PerformanceObserver((entryList) => {
          const entries = entryList.getEntries();
          if (entries.length > 0) {
            // Necesitamos especificar que este es un PerformanceEventTiming
            const fid = entries[0] as PerformanceEventTiming;
            console.log('FID:', Math.round(fid.processingStart - fid.startTime), 'ms');
          }
        }).observe({ type: 'first-input', buffered: true });
        
        // CLS
        let clsValue = 0;
        new PerformanceObserver((entryList) => {
          const entries = entryList.getEntries();
          entries.forEach(entry => {
            // Necesitamos especificar que este es un LayoutShift
            const layoutShift = entry as LayoutShift;
            if (!layoutShift.hadRecentInput) {
              clsValue += layoutShift.value;
            }
          });
          console.log('CLS:', clsValue);
        }).observe({ type: 'layout-shift', buffered: true });
        
        // Nuevas métricas para móviles
        // Interaction to Next Paint (INP)
        if ('interactionCount' in PerformanceEventTiming.prototype) {
          new PerformanceObserver((entryList) => {
            const interactions = entryList.getEntries();
            if (interactions.length > 0) {
              const inp = interactions.reduce((max, entry) => {
                const duration = (entry as PerformanceEventTiming).processingEnd - (entry as PerformanceEventTiming).startTime;
                return duration > max ? duration : max;
              }, 0);
              console.log('INP:', Math.round(inp), 'ms');
            }
          // Using only standardized properties in PerformanceObserverInit
          }).observe({ type: 'event', buffered: true });
        }
        
      } catch (e) {
        console.error('Error midiendo Web Vitals:', e);
      }
    }
  };
  
  // Ejecutar después de la carga completa
  window.addEventListener('load', () => {
    if ('requestIdleCallback' in window) {
      (window as any).requestIdleCallback(reportWebVitals, { timeout: 2000 });
    } else {
      setTimeout(reportWebVitals, 2000);
    }
  });
}
